//  A Traffic Light using the standard pattern (R,R+Y,G,Y).
//
//  The GREEN TIME for each traffic light can be configured
//  A sensor signal may be used to shorten the GREEN TIME.
//
// version 3

#include "TrafficLight.h"


// constructor, expects name, *strip, LED positions and pixels per lamp
// ====================================================================

TrafficLight::TrafficLight() {
}

TrafficLight::TrafficLight(String name, Adafruit_NeoPixel* strip, const int *ledPos, int pixPerLamp) {

	this->name = name;
    this->strip=strip;
    for (int n=0; n<3; n++) this->ledPos[n]=ledPos[n];
    this->pixPerLamp=pixPerLamp;
    
	// default values for switching times
	times[TL_OFF	] =	    0;
    times[TL_STOP  	] =   500;
    times[TL_ALERT 	] =  1000;
    times[TL_GO    	] = 10000;
    times[TL_HALT  	] =  2000;

	// right now we do not yet have a sensor attached
	this->sensorPin = 0;

	// start in OFF STATE with full brightness
	this->startedAt=esp_timer_get_time();
	this->off();
    this->setBrightness(255);
}


// Layer (7) : add sensor, configure green time, set brightness
// ============================================================

void TrafficLight::addSensor(int sensorPin) {
    this->sensorPin = sensorPin;
    pinMode(sensorPin,INPUT);
}

void TrafficLight::setGreenTime(int greenTime) {
    times[TL_GO] = greenTime;
}

void TrafficLight::setBrightness(byte brightness) {
    debug(name+" brightness = "+String(brightness));
    colors[TL_RED_LAMP   ] = strip->Color( brightness	,   		0	,   0 );
    colors[TL_YELLOW_LAMP] = strip->Color( brightness/2	, brightness/2	,   0 );
    colors[TL_GREEN_LAMP ] = strip->Color(   	0		, brightness	,   0 );
}


// Layer (6) : walk through all states in the correct sequence
// ===========================================================

void TrafficLight::performCycle() {
    stop    ();
    alert   ();
    go      ();
    halt    ();
    stop    ();   
}


// Layer (5) : change state of TL switching the necessary lamps ON or OFF
// ======================================================================

void TrafficLight::off() {
    debug(name+" = OFF");
    clear();
    show();
}
void TrafficLight::stop() {
    debug(name+"\t= STOP");
    yellowOff   ();
    redOn       ();
    showFor(times[TL_STOP]);
}
void TrafficLight::alert() {
    debug(name+"\t= ALERT");
    yellowOn    ();
    showFor(times[TL_ALERT]);
}
void TrafficLight::go() {
    debug(name+"\t= GO");
    redOff      ();
    yellowOff   ();
    greenOn     ();
    showForMin(times[TL_GO],50);   // at least 50% of the configured time
}
void TrafficLight::halt() {
    debug(name+"\t= HALT");
    greenOff    ();
    yellowOn    ();
    showFor(times[TL_HALT]);
}


// Layer (4) : switch a certain lamp ON or OFF
// ===========================================

void TrafficLight::redOn()      { lampOn  (TL_RED_LAMP);    }
void TrafficLight::redOff()     { lampOff (TL_RED_LAMP);    }
void TrafficLight::yellowOn()   { lampOn  (TL_YELLOW_LAMP); }
void TrafficLight::yellowOff()  { lampOff (TL_YELLOW_LAMP); }
void TrafficLight::greenOn()    { lampOn  (TL_GREEN_LAMP);  }
void TrafficLight::greenOff()   { lampOff (TL_GREEN_LAMP);  }
void TrafficLight::clear()		{
    greenOff	();
    yellowOff   ();
    redOff      ();
}


// Layer (3) : switch a single lamp ON or OFF using its fixed color
// ================================================================

void TrafficLight::lampOn(int lampNr) {
    setColor(lampNr,colors[lampNr]);    // each lamp has its fixed color
}
void TrafficLight::lampOff(int lampNr) {
    setColor(lampNr,0);    // color 0 == BLACK == OFF
}


// Layer (2) : set a single lamp to an arbitrary color
// ===================================================

void TrafficLight::setColor(int lampNr, uint32_t color) {
    strip->fill(color,ledPos[lampNr],pixPerLamp);    
}


// Layer (1) :  transfer LED states to the strip, wait for the specified time
// ==========================================================================

void TrafficLight::show() {
    strip->show();
}

void TrafficLight::showFor(int msec) {
    showForMin(msec,100);
}

// allow interruption after minPct(%) is over
bool TrafficLight::showForMin(int msec, int pct) {
    show();
    bool interrupted=false;
    int tick=20;
    int minMsec = round (msec * pct / 100);
    for(int timeOver=0;timeOver<msec;timeOver+=tick) {
        delay(tick);
        if (sensorPin<=0 || pct>=100) continue;
        if (digitalRead(sensorPin)) {
			if (!interrupted) {
				debug(name+"\t: SENSOR SIGNAL DETECTED");
				interrupted=true;
			}
		}
        if (interrupted && timeOver>=minMsec) return false;        
    }
    return true;
}


// Layer (0) :  transmit debug messages with time stamps to the serial port
// ========================================================================

void TrafficLight::debug(String message) {
    Serial.println(String(round((esp_timer_get_time()-startedAt)/100000.)/10)+"\t"+message);
}
