#ifndef TRAFFIC_LIGHT_H
#define TRAFFIC_LIGHT_H

#include <Adafruit_NeoPixel.h>

class TrafficLight {

    // A traffic light with adjustable brightness.
	// The traffic light consists of three lamps.
	// Each lamp consists of one or more LED pixels.

    // symbolic constants for the different STATES of a traffic light
	#define TL_OFF					0		// all lights are off
    #define TL_STOP                	1       // red lamp on
    #define TL_ALERT               	2       // simultaneously red and yellow on
    #define TL_GO                  	3       // green lamp on
    #define TL_HALT                	4       // yellow lamp on

    // symbolic constants for the lamp color index
    #define TL_RED_LAMP     0
    #define TL_YELLOW_LAMP  1
    #define TL_GREEN_LAMP   2

    public:

        TrafficLight();
		
		// when creating a new TrafficLight we need a name, a reference to the LED strip,	
        // the LED positions (per lamp) and the number of LEDs per lamp
        TrafficLight(String name, Adafruit_NeoPixel *strip, const int *ledPos, int pixPerLamp);

        /*
         * We have several layers of functions, from TOP to BOTTOM:
         *   (7) setGreenTime, addSensor, setBrightness : configuration
         *   (6) performCycle        	: walk through all states in the correct sequence
         *   (5) off,stop,alert,go,halt : change state of TrafficLight switching appropriate lamp(s)
         *   (4) ..On, ..Off, clear    	: switch a certain lamp (x) ON or OFF, clear all lamps
         *   (3) lampOn, lampOff       	: switch a lamp on/off using its defined color
         *   (2) setColor            	: assign a color to an arbitrary lamp, changing all of its LEDs
         *   (1) showFor, showForMin 	: transfer LED settings to the strip and wait  for the appropriate time
         */

        
        void addSensor(int sensorPin);
        void setGreenTime(int time);		// in msec
        void setBrightness(byte brightness);	// 0=off, 255 is maximum

        void performCycle();
        
		void off();
        void stop();
        void alert();
        void go();
        void halt();
                
        void redOn();
        void redOff();
        void yellowOn();
        void yellowOff();
        void greenOn();
        void greenOff();
		void clear();

        void lampOn(int lampNr);			// from top to bottom, 0=red, 1=yellow, 2=green
        void lampOff(int lampNr);

        void showFor(int time);				// in msec
        bool showForMin(int time, int pct);	// show for 'time' or shorter, but at least 'pct' percent of 'time'
		void show();						// just show and return

    private:


        void setColor(int lampNr, uint32_t color);	// we use the lower 24 bits as RRGGBB color
	
	    // message logging to the serial port with time stamp
        int64_t startedAt;
        void debug(String message);
        
		String name;
		Adafruit_NeoPixel *strip;
        int times[TL_HALT+1];  				// specific times per STATE in msec
        int ledPos[TL_GREEN_LAMP+1];		// first LED of each lamp
        int pixPerLamp;     				// number of pixels per lamp
        int sensorPin;      				// optional pin for traffic detection sensor
        uint32_t colors[TL_GREEN_LAMP+1];	// the three base colors; brightness may be adjusted separately
};

#endif
